/*
 * Spiral Staircase Dismount v0.2 by Heikki Kernen
 *
 * Based on the test level by Jetro Lauha 
 *
 * Changes in 0.2:
 * - Reduced the number of objects in the level by almost 3/4
 * - Adjusted gravity on the stairs
 * - Changed scoring to take into account the number of stairs hit
 *
 * Changes in 0.1:
 * - First public release
 *
 */

const int SCRIPT_GAME_VERSION = int(0x000400);

const int MAX_TIME = 30000;
int STAIR_SCORE = 300;

float SCREEN_CENTER_X;
float SCREEN_CENTER_Y;

uint mTime;
uint startTime;
int screenWidth, screenHeight;

float xx, yy = 0.0f;
float finishX, finishZ;
float startX;

int timeLeft = 0;
int score = 0;
int uniqueStairs = 0;
int stairScore = 0;

UIFont *medFont, *smallFont;
uint medFontHeight, smallFontHeight;

PhysicsObject *head, *neck, *trunk, *rthigh, *lthigh, *rcalf, *lcalf, *rfore, *lfore, *larm, *rarm, *rfoot, *lfoot;

PhysicsObject *col1, *col2;

uint id = 0;
uint id1, id2, id3, id4, id5, id6, id7, id8, id9, id10;

bool finished = false;
bool aborted = false;
bool started = false;
bool crossedLine = false;
bool stairsTouched = false;

// Called once on level initialization.
void onInit()
{
	screenWidth = getScreenWidth();
	screenHeight = getScreenHeight();

	SCREEN_CENTER_X = float(screenWidth / 2);
	SCREEN_CENTER_Y = float(screenHeight / 2);

	medFont = getFont("medium");
	smallFont = getFont("small");
	medFontHeight = medFont->getFontHeight();
	smallFontHeight = smallFont->getFontHeight();

	setGravity(0,0,-1.5);

	head = findObject("Head");
	neck = findObject("Neck");
	trunk = findObject("Trunk");
	rthigh = findObject("RightThigh");
	lthigh = findObject("LeftThigh");
	rarm = findObject("RightArm");
	larm = findObject("LeftArm");
	rfore = findObject("RightForearm");
	lfore = findObject("LeftForearm");
	rcalf = findObject("RightCalf");
	lcalf = findObject("LeftCalf");
	rfoot = findObject("RightFoot");
	lfoot = findObject("LeftFoot");

	float x,y,z = 0;
	findObject("FinishLine")->getPosition(x,y,z);
	finishX = x;
	finishZ = z - 5.0;
	findObject("StartLine")->getPosition(x,y,z);
	startX = x;

	revertColors();
}

void revertColors() {
	head->setColor(1,0.7,0.5);
	neck->setColor(1,0.7,0.5);
	trunk->setColor(1,1,0);
	rthigh->setColor(1,1,1);
	lthigh->setColor(1,1,1);
	rarm->setColor(1,1,0);
	larm->setColor(1,1,0);
	rfore->setColor(1,0.7,0.5);
	lfore->setColor(1,0.7,0.5);
	rcalf->setColor(1,0.7,0.5);
	lcalf->setColor(1,0.7,0.5);
	rfoot->setColor(0.4,0.1,0.1);
	lfoot->setColor(0.4,0.1,0.1);
}


// Called once for each game logic update before the physics update.
// Parameter 'time' is the simulation time in milliseconds.
void onUpdate(uint time)
{
	mTime = time;
	checkTime(time);
	checkDistance();
	revertColors();
}

// Called when the game view is rendered.
// Might happen more or less frequently than update calls.
void onRender()
{

	if (mTime < 5000) {
		float a = 1.0;
		if (mTime > 4000) a = float(5000-mTime) / 1000.0;
		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y / 3, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Spiral Staircase Dismount v0.2", 1.0,1.0,1.0,a);
		smallFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y / 3 + 30, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Get downstairs as fast as you can!", 1.0,1.0,1.0,a);
	}

	drawTime();
	if (aborted) {
		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "You failed!", 0.0,0.0,0.0,1.0);
		medFont->drawString(SCREEN_CENTER_X+1, SCREEN_CENTER_Y+1, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "You failed!", 1.0,0.0,0.0,1.0);

	}

	if (finished) {
	
		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "You made it!", 0.0,0.0,0.0,1.0);
		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y + 30, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Time score: " + bstrFormat(timeLeft), 0.0,0.0,0.0,1.0);
		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y + 60, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER,  "Stairs hit: " + bstrFormat(uniqueStairs) + " x " + bstrFormat(STAIR_SCORE) + " = " + bstrFormat(stairScore),0.0,0.0,0.0,1.0);
		medFont->drawString(SCREEN_CENTER_X, SCREEN_CENTER_Y + 90, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER,  "Total score: " + bstrFormat(score), 0.0,0.0,0.0,1.0);

		medFont->drawString(SCREEN_CENTER_X + 1, SCREEN_CENTER_Y + 1, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "You made it!", 0,1.0,0,1.0);
		medFont->drawString(SCREEN_CENTER_X + 1, SCREEN_CENTER_Y + 31, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER, "Time score: " + bstrFormat(timeLeft), 1.0,1.0,1.0,1.0);
		medFont->drawString(SCREEN_CENTER_X + 1, SCREEN_CENTER_Y + 61, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER,  "Stairs hit: " + bstrFormat(uniqueStairs) + " x " + bstrFormat(STAIR_SCORE) +" = " + bstrFormat(stairScore),1.0,1.0,1.0,1.0);
		medFont->drawString(SCREEN_CENTER_X + 1, SCREEN_CENTER_Y + 91, UIFONT_ALIGN_HORIZ_CENTER | UIFONT_ALIGN_VERT_CENTER,  "Total score: " + bstrFormat(score),1.0,1.0,1.0,1.0);

	}

}
// Called when there is a collision between the two given objects.
// Returns true to create contacts or false to ignore collision.
bool onCollision(PhysicsObject *o1, PhysicsObject *o2)
{
	col1 = o1;
	col2 = o2;

	if (playerHit("GroundFloor")) {
		setGravity(0,0,-1.5);
		if (crossedLine && stairsTouched) {
			finished = true;
			setGameState(GAMESTATE_FINISHED);
			calculateScore();
		}
	}

	if (o1->getBlockColor() == 1) {
		o2->setColor(1,0,0);
		stairCollision(o1);
	} else if (o2->getBlockColor() == 1) {
		o1->setColor(1,0,0);
		stairCollision(o2);
	}

	// generate collision
	return true;
}

bool playerHit(bstr name) {
	return ((col1->getType() == PHYSOBJ_RAGDOLL_BODY && col2->getName() == name) || (col2->getType() == PHYSOBJ_RAGDOLL_BODY && col1->getName() == name));
}

void checkTime(uint time) {
	if (started && !finished && !aborted && MAX_TIME != 0)
	{
		timeLeft = int(MAX_TIME) + int(startTime - time);
		if (timeLeft < 0) timeLeft = 0;
		if (timeLeft == 0 && !aborted)
		{
			aborted = true;
			setGameState(GAMESTATE_ABORTED);
		}
	}
}

void stairCollision(PhysicsObject *obj) {
	// magic
	id = obj->getId();
	xx = float(int(id/1000 - 300)) / 300.0;
	yy = -float(int(id % 1000 - 300)) / 300.0;
	setGravity(xx,yy,-3);
	stairsTouched = true;
	checkNewStair(id);
}

void checkNewStair(uint id) {
	if (id != id1 && id != id2 && id != id3 && id != id4 && id != id5 && id != id6 && id != id7 && id != id8 && id != id9 && id != id10) {
		id10 = id9;
		id9 = id8;
		id8 = id7;
		id7 = id6;
		id6 = id5;
		id5 = id4;
		id4 = id3;
		id3 = id2;
		id2 = id1;
		id1 = id;
		uniqueStairs++;
	}
}

void checkDistance() {
	float x, y, z = 0;
	head->getPosition(x, y, z);

	if (!started && x>startX) {
		started = true;
		startTime = mTime;
	}

	if (!aborted && z < finishZ)
	{
		aborted = true;
		setGameState(GAMESTATE_ABORTED);
	}

	if (x > finishX) crossedLine = true;
}

void calculateScore() {
	score = timeLeft;
	stairScore = uniqueStairs * STAIR_SCORE;
	score += stairScore;
}

void drawTime()
{
	const float x = 100, y = 70;
	const float gap = 40;
	int fracs = (timeLeft / 10) % 100;
	int seconds = timeLeft / 1000;
	bstr fracsStr = (fracs < 10 ? "0" : "") + bstrFormat(fracs);

	medFont->drawString(x, y, UIFONT_ALIGN_HORIZ_CENTER, ":");
	medFont->drawString(x - 4, y, UIFONT_ALIGN_RIGHT, bstrFormat(seconds));
	medFont->drawString(x + 4, y, UIFONT_ALIGN_DEFAULT, fracsStr);
	smallFont->drawString(x, y + float(medFontHeight), UIFONT_ALIGN_HORIZ_CENTER, "Time Left");

	medFont->drawString(x, y + gap, UIFONT_ALIGN_HORIZ_CENTER, bstrFormat(uniqueStairs));
	smallFont->drawString(x, y + float(medFontHeight) + gap, UIFONT_ALIGN_HORIZ_CENTER, "Stairs hit");

}
